@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2017 Red Hat, Inc.
+// Copyright (C) 2013-2018 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -578,9 +578,8 @@ main(int argc, char* argv[])
 
   if (opts.display_version)
     {
-      string major, minor, revision;
-      abigail::abigail_get_library_version(major, minor, revision);
-      cout << major << "." << minor << "." << revision << "\n";
+      emit_prefix(argv[0], cout)
+	<< abigail::tools_utils::get_library_version_string();
       return 0;
     }
 
