@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2015-2017 Red Hat, Inc.
+// Copyright (C) 2015-2018 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -95,6 +95,7 @@
 #include <vector>
 #include <algorithm>
 #include <map>
+#include <tr1/unordered_set>
 
 #include "abg-workers.h"
 #include "abg-config.h"
@@ -109,6 +110,7 @@ using std::string;
 using std::ostream;
 using std::vector;
 using std::map;
+using std::tr1::unordered_set;
 using std::ostringstream;
 using std::tr1::shared_ptr;
 using std::tr1::dynamic_pointer_cast;
@@ -123,6 +125,10 @@ using abigail::tools_utils::check_file;
 using abigail::tools_utils::ensure_dir_path_created;
 using abigail::tools_utils::guess_file_type;
 using abigail::tools_utils::string_ends_with;
+using abigail::tools_utils::dir_name;
+using abigail::tools_utils::real_path;
+using abigail::tools_utils::string_suffix;
+using abigail::tools_utils::sorted_strings_common_prefix;
 using abigail::tools_utils::file_type;
 using abigail::tools_utils::make_path_absolute;
 using abigail::tools_utils::base_name;
@@ -171,8 +177,8 @@ public:
   bool		parallel;
   string	package1;
   string	package2;
-  string	debug_package1;
-  string	debug_package2;
+  vector<string> debug_packages1;
+  vector<string> debug_packages2;
   string	devel_package1;
   string	devel_package2;
   size_t	num_workers;
@@ -233,6 +239,12 @@ public:
   }
 };
 
+static bool
+get_interesting_files_under_dir(const string	dir,
+				const string&	file_name_to_look_for,
+				options&	opts,
+				vector<string>& interesting_files);
+
 /// Abstract ELF files from the packages which ABIs ought to be
 /// compared
 class elf_file
@@ -318,13 +330,15 @@ public:
 private:
   string				path_;
   string				extracted_dir_path_;
+  string				common_paths_prefix_;
   abigail::tools_utils::file_type	type_;
   kind					kind_;
   map<string, elf_file_sptr>		path_elf_file_sptr_map_;
-  package_sptr				debug_info_package_;
+  vector<package_sptr>			debug_info_packages_;
   package_sptr				devel_package_;
   package_sptr				kabi_whitelist_package_;
   suppressions_type			private_types_suppressions_;
+  vector<string>			elf_file_paths_;
 
 public:
   /// Constructor for the @ref package type.
@@ -398,6 +412,31 @@ public:
   extracted_dir_path(const string& p)
   {extracted_dir_path_ = p;}
 
+  /// Getter of the the prefix that is common to all the paths of all
+  /// the elements of the package.
+  ///
+  /// @return the common path prefix of package elements.
+  const string&
+  common_paths_prefix() const
+  {return common_paths_prefix_;}
+
+  /// Getter of the the prefix that is common to all the paths of all
+  /// the elements of the package.
+  ///
+  /// @return the common path prefix of package elements.
+  string&
+  common_paths_prefix()
+  {return common_paths_prefix_;}
+
+  /// Setter of the the prefix that is common to all the paths of all
+  /// the elements of the package.
+  ///
+  ///
+  ///@param p the new prefix. 
+  void
+  common_paths_prefix(const string& p)
+  {common_paths_prefix_ = p;}
+
   /// Getter for the file type of the current package.
   ///
   /// @return the file type of the current package.
@@ -439,22 +478,43 @@ public:
   path_elf_file_sptr_map()
   {return path_elf_file_sptr_map_;}
 
-  /// Getter for the debug info package associated to the current
+  /// Getter for the debug info packages associated to the current
   /// package.
   ///
-  /// @return the debug info package associated to the current
+  /// There can indeed be several debug info packages needed for one
+  /// input package, as the debug info for that input package can be
+  /// split across several debuginfo packages.
+  ///
+  /// @return the debug info packages associated to the current
   /// package.
-  const shared_ptr<package>&
-  debug_info_package() const
-  {return debug_info_package_;}
+  const vector<package_sptr>&
+  debug_info_packages() const
+  {return debug_info_packages_;}
 
-  /// Setter for the debug info package associated to the current
+  /// Getter for the debug info packages associated to the current
   /// package.
   ///
+  /// There can indeed be several debug info packages needed for one
+  /// input package, as the debug info for that input package can be
+  /// split across several debuginfo packages.
+  ///
+  /// @return the debug info packages associated to the current
+  /// package.
+  vector<package_sptr>&
+  debug_info_packages()
+  {return debug_info_packages_;}
+
+  /// Setter for the debug info packages associated to the current
+  /// package.
+  ///
+  /// There can indeed be several debug info packages needed for one
+  /// input package, as the debug info for that input package can be
+  /// split across several debuginfo packages.
+  ///
   /// @param p the new debug info package.
   void
-  debug_info_package(const shared_ptr<package> p)
-  {debug_info_package_ = p;}
+  debug_info_packages(const vector<package_sptr> &p)
+  {debug_info_packages_ = p;}
 
   /// Getter for the devel package associated to the current package.
   ///
@@ -502,6 +562,86 @@ public:
   private_types_suppressions()
   {return private_types_suppressions_;}
 
+  /// Getter of the path to the elf files of the package.
+  ///
+  /// @return the path tothe elf files of the package.
+  const vector<string>&
+  elf_file_paths() const
+  {return elf_file_paths_;}
+
+  /// Getter of the path to the elf files of the package.
+  ///
+  /// @return the path tothe elf files of the package.
+  vector<string>&
+  elf_file_paths()
+  {return elf_file_paths_;}
+
+  /// Convert the absolute path of an element of this package into a
+  /// path relative to the root path pointing to this package.
+  ///
+  /// That is, suppose the content of a package named 'pkg' is located
+  /// at /root/path/pkg.  Suppose an element of that package is named
+  /// is at '/root/path/pkg/somewhere/inside/element'.
+  ///
+  /// This function will return the path:
+  /// /pkg/somewhere/inside/element.
+  ///
+  /// @param path the path to consider.
+  ///
+  /// @param converted_path the resulting converted path.  This is set
+  /// iff the function returns true.
+  ///
+  /// @return true if the path could be converted to being relative to
+  /// the extracted directory.
+  bool
+  convert_path_to_relative(const string& path, string& converted_path) const
+  {
+    string root = extracted_dir_path_;
+    real_path(root, root);
+    string p = path;
+    real_path(p, p);
+    return string_suffix(p, root, converted_path);
+  }
+
+  // Convert the absolute path of an element of this package into a
+  // path relative to the prefix common to the paths of all elements
+  // of the package.
+  //
+  // @param path the path to conver.
+  //
+  // @param converted_path the resulting converted path.  This is set
+  // iff the function returns true.
+  //
+  // @return true iff the function could successfully convert @p path
+  // and put the result into @p converted_path.
+  bool
+  convert_path_to_unique_suffix(const string& path,
+				string& converted_path) const
+  {return string_suffix(path, common_paths_prefix(), converted_path);}
+
+  /// Retrieve the set of "interesting" package element paths by
+  /// walking the package.
+  ///
+  /// And then compute the path prefix that is common to all the
+  /// collected elements.
+  ///
+  /// @param the options of this application.
+  void
+  load_elf_file_paths(options& opts)
+  {
+    if (!common_paths_prefix().empty()
+	|| !elf_file_paths().empty())
+      // We have already loaded the elf file paths, don't do it again.
+      return;
+
+    get_interesting_files_under_dir(extracted_dir_path(),
+				    /*file_name_to_look_for=*/"",
+				    opts, elf_file_paths());
+    std::sort(elf_file_paths().begin(), elf_file_paths().end());
+    string common_prefix;
+    sorted_strings_common_prefix(elf_file_paths(), common_paths_prefix());
+  }
+
   /// Erase the content of the temporary extraction directory that has
   /// been populated by the @ref extract_package() function;
   ///
@@ -541,8 +681,8 @@ public:
   erase_extraction_directories(const options &opts) const
   {
     erase_extraction_directory(opts);
-    if (debug_info_package())
-      debug_info_package()->erase_extraction_directory(opts);
+    if (!debug_info_packages().empty())
+      debug_info_packages().front()->erase_extraction_directory(opts);
     if (devel_package())
       devel_package()->erase_extraction_directory(opts);
     if (kabi_whitelist_package())
@@ -591,7 +731,7 @@ struct compare_args
 /// A convenience typedef for arguments passed to the comparison workers.
 typedef shared_ptr<compare_args> compare_args_sptr;
 
-static bool extract_package_and_map_its_content(package &pkg,
+static bool extract_package_and_map_its_content(const package_sptr &pkg,
 						options &opts);
 
 /// Getter for the path to the parent directory under which packages
@@ -727,17 +867,8 @@ extract_rpm(const string& package_path,
       << extracted_package_dir_path
       << " ...";
 
-  string cmd = "test -d " +
-    extracted_package_dir_path +
-    " && rm -rf " + extracted_package_dir_path;
-
-  if (system(cmd.c_str()))
-    {
-      if (opts.verbose)
-	emit_prefix("abipkgdiff", cerr) << "command " << cmd << " FAILED\n";
-    }
-
-  cmd = "mkdir -p " + extracted_package_dir_path + " && cd " +
+  string cmd = "test -d " + extracted_package_dir_path
+    + " || mkdir -p " + extracted_package_dir_path + " ; cd " +
     extracted_package_dir_path + " && rpm2cpio " + package_path +
     " | cpio -dium --quiet";
 
@@ -781,17 +912,7 @@ extract_deb(const string& package_path,
       << extracted_package_dir_path
       << " ...";
 
-  string cmd = "test -d " +
-    extracted_package_dir_path +
-    " && rm -rf " + extracted_package_dir_path;
-
-  if (system(cmd.c_str()))
-    {
-      if (opts.verbose)
-	emit_prefix("abipkgdiff", cerr) << "command "  << cmd <<  " FAILED\n";
-    }
-
-  cmd = "mkdir -p " + extracted_package_dir_path + " && dpkg -x " +
+  string cmd = "mkdir -p " + extracted_package_dir_path + " && dpkg -x " +
     package_path + " " + extracted_package_dir_path;
 
   if (system(cmd.c_str()))
@@ -1154,7 +1275,7 @@ compare(const elf_file& elf1,
 	bail_out = true;
       }
 
-    if (opts.fail_if_no_debug_info || opts.verbose)
+    if (opts.fail_if_no_debug_info)
       {
 	bool debug_info_error = false;
 	if (c1_status & abigail::dwarf_reader::STATUS_DEBUG_INFO_NOT_FOUND)
@@ -1240,7 +1361,7 @@ compare(const elf_file& elf1,
 	bail_out = true;
       }
 
-    if (opts.fail_if_no_debug_info || opts.verbose)
+    if (opts.fail_if_no_debug_info)
       {
 	bool debug_info_error = false;
 	if (c2_status & abigail::dwarf_reader::STATUS_DEBUG_INFO_NOT_FOUND)
@@ -1493,30 +1614,41 @@ maybe_handle_kabi_whitelist_pkg(const package& pkg, options &opts)
   return true;
 }
 
-/// The task that performs the extraction of the content of a package
-/// into a temporary directory.
+/// The task that performs the extraction of the content of several
+/// packages into a temporary directory.
 ///
-/// Note that several instanaces of tasks can perform their jobs in
-/// parallel.
+/// If this task has several packages to extract, then it extracts
+/// them in sequence.
+///
+/// Note that several instances of tasks can perform their jobs (i.e
+/// extract packages in sequence) in parallel.
 class pkg_extraction_task : public task
 {
   pkg_extraction_task();
 
 public:
-  package &pkg;
+  vector<package_sptr> pkgs;
   const options &opts;
   bool is_ok;
 
-  pkg_extraction_task(package &p, const options &o)
-    : pkg(p), opts(o), is_ok(false)
+  pkg_extraction_task(const package_sptr &p, const options &o)
+    : opts(o), is_ok(true)
+  {pkgs.push_back(p);}
+
+  pkg_extraction_task(const vector<package_sptr> &packages, const options &o)
+    : pkgs(packages), opts(o), is_ok(true)
   {}
 
-  /// The job performed by the current task.  It's to be performed in
-  /// parallel with other jobs.
+  /// The job performed by the current task, which is to extract its
+  /// packages in sequence.  This is job is to be performed in
+  /// parallel with other jobs of other tasks.
   virtual void
   perform()
   {
-    is_ok = extract_package(pkg, opts);
+    for (vector<package_sptr>::const_iterator p = pkgs.begin();
+	 p != pkgs.end();
+	 ++p)
+      is_ok &= extract_package(**p, opts);
   }
 }; //end class pkg_extraction_task
 
@@ -1536,11 +1668,11 @@ class pkg_prepare_task : public abigail::workers::task
   pkg_prepare_task();
 
 public:
-  package &pkg;
+  package_sptr pkg;
   options &opts;
   bool is_ok;
 
-  pkg_prepare_task(package &p, options &o)
+  pkg_prepare_task(package_sptr &p, options &o)
     : pkg(p), opts(o), is_ok(false)
   {}
 
@@ -1548,7 +1680,7 @@ public:
   virtual void
   perform()
   {
-    is_ok = extract_package_and_map_its_content(pkg, opts);
+    is_ok = pkg && extract_package_and_map_its_content(pkg, opts);
   }
 }; //end class pkg_prepare_task
 
@@ -1611,7 +1743,7 @@ public:
 	  out << "No ABI change detected\n";
       }
 
-    // If an error happened while comparing the twobinaries, tell the
+    // If an error happened while comparing the two binaries, tell the
     // user about it.
     if (status & abigail::tools_utils::ABIDIFF_ERROR)
       {
@@ -1659,10 +1791,10 @@ typedef shared_ptr<compare_task> compare_task_sptr;
 /// of the current directory tree being analyzed.  These paths are
 /// those that are going to be involved in ABI comparison.
 static void
-maybe_update_vector_of_package_content(const FTSENT *entry,
-				       options &opts,
-				       const string& file_name_to_look_for,
-				       vector<string>& paths)
+maybe_update_package_content(const FTSENT *entry,
+			     options &opts,
+			     const string& file_name_to_look_for,
+			     unordered_set<string>& paths)
 {
   if (entry == NULL
       || (entry->fts_info != FTS_F && entry->fts_info != FTS_SL)
@@ -1678,19 +1810,19 @@ maybe_update_vector_of_package_content(const FTSENT *entry,
       string name;
       abigail::tools_utils::base_name(path, name);
       if (name == file_name_to_look_for)
-	paths.push_back(path);
+	paths.insert(path);
       return;
     }
 
   if (guess_file_type(path) == abigail::tools_utils::FILE_TYPE_ELF)
-    paths.push_back(path);
+    paths.insert(path);
   else if (opts.abignore && string_ends_with(path, ".abignore"))
     opts.suppression_paths.push_back(path);
 }
 
 /// Walk a given directory to collect files that are "interesting" to
-/// analyze.  By default, "interesting" means interesting from a
-/// kernel package analysis point of view.
+/// analyze.  By default, "interesting" means interesting from either
+/// a kernel package or a userspace binary analysis point of view.
 ///
 /// @param dir the directory to walk.
 ///
@@ -1709,17 +1841,26 @@ get_interesting_files_under_dir(const string dir,
 				vector<string>& interesting_files)
 {
   bool is_ok = false;
-  char* paths[] = {const_cast<char*>(dir.c_str()), 0};
+  string root;
+  real_path(dir, root);
+  if (root.empty())
+    root = dir;
+
+  char* paths[] = {const_cast<char*>(root.c_str()), 0};
 
   FTS *file_hierarchy = fts_open(paths, FTS_LOGICAL|FTS_NOCHDIR, NULL);
   if (!file_hierarchy)
     return is_ok;
 
   FTSENT *entry;
+  unordered_set<string> files;
   while ((entry = fts_read(file_hierarchy)))
-    maybe_update_vector_of_package_content(entry, opts,
-					   file_name_to_look_for,
-					   interesting_files);
+    maybe_update_package_content(entry, opts, file_name_to_look_for, files);
+
+  for (unordered_set<string>::const_iterator i = files.begin();
+       i != files.end();
+       ++i)
+    interesting_files.push_back(*i);
 
   fts_close(file_hierarchy);
 
@@ -1781,6 +1922,11 @@ create_maps_of_package_content(package& package, options& opts)
       << "Found " << elf_file_paths.size() << " files in "
       << package.extracted_dir_path() << "\n";
 
+  // determine if all files have the same prefix.  Compute that prefix
+  // and stick it into the package!  That prefix is going to be used
+  // later by the package::convert_path_to_unique_suffix method.
+  package.load_elf_file_paths(opts);
+
   for (vector<string>::const_iterator file = elf_file_paths.begin();
        file != elf_file_paths.end();
        ++file)
@@ -1822,9 +1968,36 @@ create_maps_of_package_content(package& package, options& opts)
 	}
 
       if (e->soname.empty())
-	package.path_elf_file_sptr_map()[e->name] = e;
+	{
+	  // Several binaries at different paths can have the same
+	  // base name.  So let's consider the full path of the binary
+	  // inside the extracted directory.
+	  string key = e->name;
+	  package.convert_path_to_unique_suffix(e->path, key);
+	  package.path_elf_file_sptr_map()[key] = e;
+	  if (opts.verbose)
+	    emit_prefix("abipkgdiff", cerr)
+	      << "mapped binary with key '" << key << "'"
+	      << "\n";
+	}
       else
-	package.path_elf_file_sptr_map()[e->soname] = e;
+	{
+	  // Several binaries at different paths can have the same
+	  // soname.  So let's *also* consider the full path of the
+	  // binary inside the extracted directory, not just the
+	  // soname.
+	  string key = e->soname;
+	  if (package.convert_path_to_unique_suffix(e->path, key))
+	    {
+	      dir_name(key, key);
+	      key += string("/@soname:") + e->soname;
+	    }
+	  package.path_elf_file_sptr_map()[key] = e;
+	  if (opts.verbose)
+	    emit_prefix("abipkgdiff", cerr)
+	      << "mapped binary with key '" << key << "'"
+	      << "\n";
+	}
     }
 
   if (opts.verbose)
@@ -1851,8 +2024,9 @@ create_maps_of_package_content(package& package, options& opts)
 ///
 /// @return true iff the extraction and analyzing went well.
 static bool
-extract_package_and_map_its_content(package &pkg, options &opts)
+extract_package_and_map_its_content(const package_sptr &pkg, options &opts)
 {
+  assert(pkg);
 
   pkg_extraction_task_sptr main_pkg_extraction;
   pkg_extraction_task_sptr dbg_extraction;
@@ -1863,21 +2037,22 @@ extract_package_and_map_its_content(package &pkg, options &opts)
 
   main_pkg_extraction.reset(new pkg_extraction_task(pkg, opts));
 
-  if (package_sptr dbg_pkg = pkg.debug_info_package())
+  if (!pkg->debug_info_packages().empty())
     {
-      dbg_extraction.reset(new pkg_extraction_task(*dbg_pkg, opts));
+      dbg_extraction.reset(new pkg_extraction_task(pkg->debug_info_packages(),
+						   opts));
       ++NUM_EXTRACTIONS;
     }
 
-  if (package_sptr devel_pkg = pkg.devel_package())
+  if (package_sptr devel_pkg = pkg->devel_package())
     {
-      devel_extraction.reset(new pkg_extraction_task(*devel_pkg, opts));
+      devel_extraction.reset(new pkg_extraction_task(devel_pkg, opts));
       ++NUM_EXTRACTIONS;
     }
 
-  if (package_sptr kabi_wl_pkg = pkg.kabi_whitelist_package())
+  if (package_sptr kabi_wl_pkg = pkg->kabi_whitelist_package())
     {
-      kabi_whitelist_extraction.reset(new pkg_extraction_task(*kabi_wl_pkg,
+      kabi_whitelist_extraction.reset(new pkg_extraction_task(kabi_wl_pkg,
 							      opts));
       ++NUM_EXTRACTIONS;
     }
@@ -1899,12 +2074,12 @@ extract_package_and_map_its_content(package &pkg, options &opts)
   // Analyze and map the content of the extracted package.
   bool is_ok = false;
   if (main_pkg_extraction->is_ok)
-    is_ok = create_maps_of_package_content(pkg, opts);
+    is_ok = create_maps_of_package_content(*pkg, opts);
 
   if (is_ok)
     {
-      maybe_create_private_types_suppressions(pkg, opts);
-      maybe_handle_kabi_whitelist_pkg(pkg, opts);
+      maybe_create_private_types_suppressions(*pkg, opts);
+      maybe_handle_kabi_whitelist_pkg(*pkg, opts);
     }
 
   return is_ok;
@@ -1928,7 +2103,9 @@ extract_package_and_map_its_content(package &pkg, options &opts)
 ///
 /// @return true iff the preparation went well.
 static bool
-prepare_packages(package &first_package, package &second_package, options &opts)
+prepare_packages(package_sptr &first_package,
+		 package_sptr &second_package,
+		 options &opts)
 {
   pkg_prepare_task_sptr first_pkg_prepare;
   pkg_prepare_task_sptr second_pkg_prepare;
@@ -2070,16 +2247,15 @@ compare_prepared_userspace_packages(package& first_package,
 
   // Setting debug-info path of libraries
   string debug_dir1, debug_dir2, relative_debug_path = "/usr/lib/debug/";
-  if (first_package.debug_info_package()
-      && second_package.debug_info_package())
+  if (!first_package.debug_info_packages().empty()
+      && !second_package.debug_info_packages().empty())
     {
       debug_dir1 =
-	first_package.debug_info_package()->extracted_dir_path() +
+	first_package.debug_info_packages().front()->extracted_dir_path() +
+	relative_debug_path;
+      debug_dir2 =
+	second_package.debug_info_packages().front()->extracted_dir_path() +
 	relative_debug_path;
-      if (second_package.debug_info_package())
-	debug_dir2 =
-	  second_package.debug_info_package()->extracted_dir_path() +
-	  relative_debug_path;
     }
 
   for (map<string, elf_file_sptr>::iterator it =
@@ -2172,7 +2348,9 @@ compare_prepared_userspace_packages(package& first_package,
       for (vector<elf_file_sptr>::iterator it = diff.removed_binaries.begin();
 	   it != diff.removed_binaries.end(); ++it)
 	{
-	  cout << "  " << (*it)->name << ", ";
+	  string relative_path;
+	  first_package.convert_path_to_relative((*it)->path, relative_path);
+	  cout << "  " << relative_path << ", ";
 	  string soname;
 	  get_soname_of_elf_file((*it)->path, soname);
 	  if (!soname.empty())
@@ -2190,7 +2368,9 @@ compare_prepared_userspace_packages(package& first_package,
       for (vector<elf_file_sptr>::iterator it = diff.added_binaries.begin();
 	   it != diff.added_binaries.end(); ++it)
 	{
-	  cout << "  " << (*it)->name << ", ";
+	  string relative_path;
+	  second_package.convert_path_to_relative((*it)->path, relative_path);
+	  cout << "  " << relative_path << ", ";
 	  string soname;
 	  get_soname_of_elf_file((*it)->path, soname);
 	  if (!soname.empty())
@@ -2238,16 +2418,15 @@ compare_prepared_linux_kernel_packages(package& first_package,
 
   // Setting debug-info path of binaries
   string debug_dir1, debug_dir2, relative_debug_path = "/usr/lib/debug/";
-  if (first_package.debug_info_package()
-      && second_package.debug_info_package())
+  if (!first_package.debug_info_packages().empty()
+      && !second_package.debug_info_packages().empty())
     {
       debug_dir1 =
-	first_package.debug_info_package()->extracted_dir_path() +
+	first_package.debug_info_packages().front()->extracted_dir_path() +
+	relative_debug_path;
+      debug_dir2 =
+	second_package.debug_info_packages().front()->extracted_dir_path() +
 	relative_debug_path;
-      if (second_package.debug_info_package())
-	debug_dir2 =
-	  second_package.debug_info_package()->extracted_dir_path() +
-	  relative_debug_path;
     }
 
   string vmlinux_path1, vmlinux_path2;
@@ -2369,7 +2548,7 @@ compare_prepared_package(package& first_package, package& second_package,
 ///
 /// @return the status of the comparison.
 static abidiff_status
-compare(package& first_package, package& second_package,
+compare(package_sptr& first_package, package_sptr& second_package,
 	abi_diff& diff, options& opts)
 {
   // Prepare (extract and analyze the contents) the packages and their
@@ -2378,11 +2557,11 @@ compare(package& first_package, package& second_package,
   // Note that the package preparations happens in parallel.
   if (!prepare_packages(first_package, second_package, opts))
     {
-      maybe_erase_temp_dirs(first_package, second_package, opts);
+      maybe_erase_temp_dirs(*first_package, *second_package, opts);
       return abigail::tools_utils::ABIDIFF_ERROR;
     }
 
-  return compare_prepared_package(first_package, second_package, diff, opts);
+  return compare_prepared_package(*first_package, *second_package, diff, opts);
 }
 
 /// Compare the ABI of two packages.
@@ -2395,7 +2574,9 @@ compare(package& first_package, package& second_package,
 ///
 /// @return the status of the comparison.
 static abidiff_status
-compare(package& first_package, package& second_package, options& opts)
+compare(package_sptr& first_package,
+	package_sptr& second_package,
+	options& opts)
 {
   abi_diff diff;
   return compare(first_package, second_package, diff, opts);
@@ -2453,8 +2634,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	      opts.wrong_option = argv[i];
 	      return true;
             }
-          opts.debug_package1 =
-	    abigail::tools_utils::make_path_absolute(argv[j]).get();
+          opts.debug_packages1.push_back
+	    (abigail::tools_utils::make_path_absolute(argv[j]).get());
           ++i;
         }
       else if (!strcmp(argv[i], "--debug-info-pkg2")
@@ -2467,8 +2648,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	      opts.wrong_option = argv[i];
 	      return true;
             }
-          opts.debug_package2 =
-	    abigail::tools_utils::make_path_absolute(argv[j]).get();
+          opts.debug_packages2.push_back
+	    (abigail::tools_utils::make_path_absolute(argv[j]).get());
           ++i;
         }
       else if (!strcmp(argv[i], "--devel-pkg1")
@@ -2664,9 +2845,8 @@ main(int argc, char* argv[])
 
   if (opts.display_version)
     {
-      string major, minor, revision;
-      abigail::abigail_get_library_version(major, minor, revision);
-      cout << major << "." << minor << "." << revision << "\n";
+      emit_prefix(argv[0], cout)
+	<< abigail::tools_utils::get_library_version_string();
       return 0;
     }
 
@@ -2700,15 +2880,19 @@ main(int argc, char* argv[])
 
   package_sptr second_package(new package(opts.package2, "package2"));
 
-  if (!opts.debug_package1.empty())
-    first_package->debug_info_package
-      (package_sptr(new package(opts.debug_package1,
+  for (vector<string>::const_iterator p = opts.debug_packages1.begin();
+       p != opts.debug_packages1.end();
+       ++p)
+    first_package->debug_info_packages().push_back
+      (package_sptr(new package(*p,
 				"debug_package1",
 				/*pkg_kind=*/package::KIND_DEBUG_INFO)));
 
-  if (!opts.debug_package2.empty())
-    second_package->debug_info_package
-      (package_sptr(new package(opts.debug_package2,
+  for (vector<string>::const_iterator p = opts.debug_packages2.begin();
+       p != opts.debug_packages2.end();
+       ++p)
+    second_package->debug_info_packages().push_back
+      (package_sptr(new package(*p,
 				"debug_package2",
 				/*pkg_kind=*/package::KIND_DEBUG_INFO)));
 
@@ -2770,8 +2954,8 @@ main(int argc, char* argv[])
 		      | abigail::tools_utils::ABIDIFF_ERROR);
 	    }
 
-	  if (!first_package->debug_info_package()
-	      || !second_package->debug_info_package())
+	  if (first_package->debug_info_packages().empty()
+	      || second_package->debug_info_packages().empty())
 	    {
 	      emit_prefix("abipkgdiff", cerr)
 		<< "a Linux Kernel package must be accompanied with its "
@@ -2834,5 +3018,5 @@ main(int argc, char* argv[])
 	      | abigail::tools_utils::ABIDIFF_ERROR);
     }
 
-  return compare(*first_package, *second_package, opts);
+  return compare(first_package, second_package, opts);
 }
